@@ -34,14 +34,14 @@ static GdkGC *gdk_pixmap_create_gc      (GdkDrawable     *drawable,
                                          GdkGCValuesMask  mask);
 static void   gdk_pixmap_draw_rectangle (GdkDrawable     *drawable,
 					 GdkGC           *gc,
-					 gint             filled,
+					 gboolean         filled,
 					 gint             x,
 					 gint             y,
 					 gint             width,
 					 gint             height);
 static void   gdk_pixmap_draw_arc       (GdkDrawable     *drawable,
 					 GdkGC           *gc,
-					 gint             filled,
+					 gboolean         filled,
 					 gint             x,
 					 gint             y,
 					 gint             width,
@@ -50,7 +50,7 @@ static void   gdk_pixmap_draw_arc       (GdkDrawable     *drawable,
 					 gint             angle2);
 static void   gdk_pixmap_draw_polygon   (GdkDrawable     *drawable,
 					 GdkGC           *gc,
-					 gint             filled,
+					 gboolean         filled,
 					 GdkPoint        *points,
 					 gint             npoints);
 static void   gdk_pixmap_draw_text      (GdkDrawable     *drawable,
@@ -233,7 +233,7 @@ gdk_pixmap_create_gc (GdkDrawable     *drawable,
 static void
 gdk_pixmap_draw_rectangle (GdkDrawable *drawable,
 			   GdkGC       *gc,
-			   gint         filled,
+			   gboolean     filled,
 			   gint         x,
 			   gint         y,
 			   gint         width,
@@ -248,7 +248,7 @@ gdk_pixmap_draw_rectangle (GdkDrawable *drawable,
 static void
 gdk_pixmap_draw_arc (GdkDrawable *drawable,
 		     GdkGC       *gc,
-		     gint         filled,
+		     gboolean     filled,
 		     gint         x,
 		     gint         y,
 		     gint         width,
@@ -266,7 +266,7 @@ gdk_pixmap_draw_arc (GdkDrawable *drawable,
 static void
 gdk_pixmap_draw_polygon (GdkDrawable *drawable,
 			 GdkGC       *gc,
-			 gint         filled,
+			 gboolean     filled,
 			 GdkPoint    *points,
 			 gint         npoints)
 {
