@@ -25,6 +25,9 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#define LINE_ATTRIBUTES (GDK_GC_LINE_WIDTH|GDK_GC_LINE_STYLE| \
+			 GDK_GC_CAP_STYLE|GDK_GC_JOIN_STYLE)
+
 #include <string.h>
 
 #include "gdkgc.h"
@@ -96,8 +99,8 @@ gdk_gc_win32_finalize (GObject *object)
 {
   GdkGCWin32 *win32_gc = GDK_GC_WIN32 (object);
   
-  if (win32_gc->clip_region)
-    gdk_region_destroy (win32_gc->clip_region);
+  if (win32_gc->hcliprgn != NULL)
+    DeleteObject (win32_gc->hcliprgn);
   
   if (win32_gc->values_mask & GDK_GC_FONT)
     gdk_font_unref (win32_gc->font);
@@ -150,7 +153,7 @@ gdk_win32_gc_values_to_win32values (GdkGCValues    *values,
 	{
 	  gdk_font_ref (win32_gc->font);
 	  win32_gc->values_mask |= GDK_GC_FONT;
-	  GDK_NOTE (GC, (g_print ("%sfont=%ps", s, win32_gc->font),
+	  GDK_NOTE (GC, (g_print ("%sfont=%p", s, win32_gc->font),
 			 s = ","));
 	}
       else
@@ -193,7 +196,8 @@ gdk_win32_gc_values_to_win32values (GdkGCValues    *values,
     {
       win32_gc->fill_style = values->fill;
       win32_gc->values_mask |= GDK_GC_FILL;
-      GDK_NOTE (GC, (g_print ("%sfill=%d", s, win32_gc->fill_style),
+      GDK_NOTE (GC, (g_print ("%sfill=%s", s,
+			      _gdk_win32_fill_style_to_string (win32_gc->fill_style)),
 		     s = ","));
     }
 
@@ -278,22 +282,12 @@ gdk_win32_gc_values_to_win32values (GdkGCValues    *values,
 
   if (mask & GDK_GC_CLIP_MASK)
     {
-      if (win32_gc->clip_region != NULL)
-	{
-	  gdk_region_destroy (win32_gc->clip_region);
-	  win32_gc->clip_region = NULL;
-	}
-
       if (win32_gc->hcliprgn != NULL)
-	{
-	  DeleteObject (win32_gc->hcliprgn);
-	  win32_gc->hcliprgn = NULL;
-	}
+	DeleteObject (win32_gc->hcliprgn);
 
       if (values->clip_mask != NULL)
 	{
-	  win32_gc->hcliprgn =
-	    _gdk_win32_bitmap_to_region (values->clip_mask);
+	  win32_gc->hcliprgn = _gdk_win32_bitmap_to_hrgn (values->clip_mask);
 	  win32_gc->values_mask |= GDK_GC_CLIP_MASK;
 	}
       else
@@ -384,7 +378,7 @@ gdk_win32_gc_values_to_win32values (GdkGCValues    *values,
           }
 	  break;
 	}
-      GDK_NOTE (GC, (g_print ("%sps|=PS_STYLE_%s", s, gdk_win32_psstyle_to_string (win32_gc->pen_style)),
+      GDK_NOTE (GC, (g_print ("%sps|=PS_STYLE_%s", s, _gdk_win32_psstyle_to_string (win32_gc->pen_style)),
 		     s = ","));
       win32_gc->values_mask |= GDK_GC_LINE_STYLE;
     }
@@ -405,7 +399,7 @@ gdk_win32_gc_values_to_win32values (GdkGCValues    *values,
 	  win32_gc->pen_style |= PS_ENDCAP_SQUARE;
 	  break;
 	}
-      GDK_NOTE (GC, (g_print ("%sps|=PS_ENDCAP_%s", s, gdk_win32_psendcap_to_string (win32_gc->pen_style)),
+      GDK_NOTE (GC, (g_print ("%sps|=PS_ENDCAP_%s", s, _gdk_win32_psendcap_to_string (win32_gc->pen_style)),
 		     s = ","));
       win32_gc->values_mask |= GDK_GC_CAP_STYLE;
     }
@@ -425,11 +419,11 @@ gdk_win32_gc_values_to_win32values (GdkGCValues    *values,
 	  win32_gc->pen_style |= PS_JOIN_BEVEL;
 	  break;
 	}
-      GDK_NOTE (GC, (g_print ("%sps|=PS_JOIN_%s", s, gdk_win32_psjoin_to_string (win32_gc->pen_style)),
+      GDK_NOTE (GC, (g_print ("%sps|=PS_JOIN_%s", s, _gdk_win32_psjoin_to_string (win32_gc->pen_style)),
 		     s = ","));
       win32_gc->values_mask |= GDK_GC_JOIN_STYLE;
     }
-  GDK_NOTE (GC, g_print ("}\n"));
+  GDK_NOTE (GC, g_print ("} mask=(%s)", _gdk_win32_gcvalues_mask_to_string (win32_gc->values_mask)));
 }
 
 GdkGC*
@@ -448,8 +442,6 @@ _gdk_win32_gc_new (GdkDrawable	  *drawable,
   gc = g_object_new (_gdk_gc_win32_get_type (), NULL);
   win32_gc = GDK_GC_WIN32 (gc);
 
-  win32_gc->hdc = NULL;
-  win32_gc->clip_region = NULL;
   win32_gc->hcliprgn = NULL;
 
   /* Use the same default values as X11 does, even if they don't make
@@ -469,13 +461,13 @@ _gdk_win32_gc_new (GdkDrawable	  *drawable,
 
   win32_gc->values_mask = GDK_GC_FUNCTION | GDK_GC_FILL;
 
-  GDK_NOTE (GC, g_print ("_gdk_win32_gc_new: "));
+  GDK_NOTE (GC, g_print ("_gdk_win32_gc_new: %p: ", win32_gc));
   gdk_win32_gc_values_to_win32values (values, mask, win32_gc);
+  GDK_NOTE (GC, g_print ("\n"));
 
+  win32_gc->hdc = NULL;
   win32_gc->hwnd = NULL;
 
-  GDK_NOTE (GC, g_print (" = %p\n", gc));
-
   return gc;
 }
 
@@ -572,9 +564,9 @@ gdk_win32_gc_set_values (GdkGC           *gc,
 {
   g_return_if_fail (GDK_IS_GC (gc));
 
-  GDK_NOTE (GC, g_print ("gdk_win32_gc_set_values: "));
-
+  GDK_NOTE (GC, g_print ("gdk_win32_gc_set_values: %p: ", GDK_GC_WIN32 (gc)));
   gdk_win32_gc_values_to_win32values (values, mask, GDK_GC_WIN32 (gc));
+  GDK_NOTE (GC, g_print ("\n"));
 }
 
 static void
@@ -614,36 +606,35 @@ gdk_gc_set_clip_rectangle (GdkGC	*gc,
   win32_gc = GDK_GC_WIN32 (gc);
 
   if (win32_gc->hcliprgn)
-    {
-      DeleteObject (win32_gc->hcliprgn);
-      win32_gc->hcliprgn = NULL;
-    }
-
-  if (win32_gc->clip_region)
-    gdk_region_destroy (win32_gc->clip_region);
+    DeleteObject (win32_gc->hcliprgn);
 
   if (rectangle)
     {
-      GDK_NOTE (GC,
-		g_print ("gdk_gc_set_clip_rectangle: %dx%d@+%d+%d\n",
-			 rectangle->width, rectangle->height,
-			 rectangle->x, rectangle->y));
-      win32_gc->clip_region = gdk_region_rectangle (rectangle);
+      GDK_NOTE (GC, g_print ("gdk_gc_set_clip_rectangle: %p: %s\n",
+			     win32_gc,
+			     _gdk_win32_gdkrectangle_to_string (rectangle)));
+      win32_gc->hcliprgn = CreateRectRgn (rectangle->x, rectangle->y,
+					  rectangle->x + rectangle->width,
+					  rectangle->y + rectangle->height);
       win32_gc->values_mask |= GDK_GC_CLIP_MASK;
     }
   else
     {
       GDK_NOTE (GC, g_print ("gdk_gc_set_clip_rectangle: NULL\n"));
 
-      win32_gc->clip_region = NULL;
+      win32_gc->hcliprgn = NULL;
       win32_gc->values_mask &= ~GDK_GC_CLIP_MASK;
     }
-    win32_gc->values_mask &= ~(GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN);
+
+  gc->clip_x_origin = 0;
+  gc->clip_y_origin = 0;
+  
+  win32_gc->values_mask &= ~(GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN);
 } 
 
 void
-gdk_gc_set_clip_region (GdkGC		 *gc,
-			GdkRegion	 *region)
+gdk_gc_set_clip_region (GdkGC	  *gc,
+			GdkRegion *region)
 {
   GdkGCWin32 *win32_gc;
 
@@ -651,30 +642,29 @@ gdk_gc_set_clip_region (GdkGC		 *gc,
 
   win32_gc = GDK_GC_WIN32 (gc);
 
-  if (win32_gc->clip_region)
-    gdk_region_destroy (win32_gc->clip_region);
+  if (win32_gc->hcliprgn)
+    DeleteObject (win32_gc->hcliprgn);
 
   if (region)
     {
-      GDK_NOTE (GC, g_print ("gdk_gc_set_clip_region: %dx%d+%d+%d\n",
-			       region->extents.x2 - region->extents.x1,
-			       region->extents.y2 - region->extents.y1,
-			       region->extents.x1, region->extents.y1));
+      GDK_NOTE (GC, g_print ("gdk_gc_set_clip_region: %p: %s\n",
+			     win32_gc,
+			     _gdk_win32_gdkregion_to_string (region)));
 
-      win32_gc->clip_region = gdk_region_copy (region);
+      win32_gc->hcliprgn = _gdk_win32_gdkregion_to_hrgn (region, 0, 0);
       win32_gc->values_mask |= GDK_GC_CLIP_MASK;
     }
   else
     {
       GDK_NOTE (GC, g_print ("gdk_gc_set_clip_region: NULL\n"));
 
-      win32_gc->clip_region = NULL;
+      win32_gc->hcliprgn = NULL;
       win32_gc->values_mask &= ~GDK_GC_CLIP_MASK;
     }
 
   gc->clip_x_origin = 0;
   gc->clip_y_origin = 0;
-
+  
   win32_gc->values_mask &= ~(GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN);
 }
 
@@ -691,24 +681,32 @@ gdk_gc_copy (GdkGC *dst_gc,
   dst_win32_gc = GDK_GC_WIN32 (dst_gc);
   src_win32_gc = GDK_GC_WIN32 (src_gc);
 
-  if (dst_win32_gc->font != NULL)
-    gdk_font_unref (dst_win32_gc->font);
-  if (dst_win32_gc->tile != NULL)
-    gdk_drawable_unref (dst_win32_gc->tile);
-  if (dst_win32_gc->stipple != NULL)
-    gdk_drawable_unref (dst_win32_gc->stipple);
-  if (dst_win32_gc->clip_region != NULL)
-    gdk_region_destroy (dst_win32_gc->clip_region);
+  GDK_NOTE (GC, g_print ("gdk_gc_copy: %p := %p\n", dst_win32_gc, src_win32_gc));
+
+  if (dst_gc->colormap)
+    g_object_unref (G_OBJECT (dst_gc->colormap));
+
   if (dst_win32_gc->hcliprgn != NULL)
     DeleteObject (dst_win32_gc->hcliprgn);
+
+  if (dst_win32_gc->font != NULL)
+    gdk_font_unref (dst_win32_gc->font);
+
+  if (dst_win32_gc->tile != NULL)
+    gdk_drawable_unref (dst_win32_gc->tile);
+
+  if (dst_win32_gc->stipple != NULL)
+    gdk_drawable_unref (dst_win32_gc->stipple);
+
   if (dst_win32_gc->pen_dashes)
     g_free (dst_win32_gc->pen_dashes);
   
   *dst_win32_gc = *src_win32_gc;
+  dst_win32_gc->hdc = NULL;
+
+  if (dst_gc->colormap)
+    g_object_ref (G_OBJECT (dst_gc->colormap));
 
-  if (dst_win32_gc->pen_dashes)
-    dst_win32_gc->pen_dashes = g_memdup (src_win32_gc->pen_dashes, 
-                                         sizeof (DWORD) * src_win32_gc->pen_num_dashes);
   if (dst_win32_gc->hcliprgn)
     {
       /* create a new region, to copy to */
@@ -717,14 +715,19 @@ gdk_gc_copy (GdkGC *dst_gc,
       CombineRgn (dst_win32_gc->hcliprgn, src_win32_gc->hcliprgn,
 		  NULL, RGN_COPY);
     }
-  if (dst_win32_gc->clip_region != NULL)
-    dst_win32_gc->clip_region = gdk_region_copy (dst_win32_gc->clip_region);
+
   if (dst_win32_gc->font != NULL)
     gdk_font_ref (dst_win32_gc->font);
+
   if (dst_win32_gc->tile != NULL)
     gdk_drawable_ref (dst_win32_gc->tile);
+
   if (dst_win32_gc->stipple != NULL)
     gdk_drawable_ref (dst_win32_gc->stipple);
+
+  if (dst_win32_gc->pen_dashes)
+    dst_win32_gc->pen_dashes = g_memdup (src_win32_gc->pen_dashes, 
+                                         sizeof (DWORD) * src_win32_gc->pen_num_dashes);
 }
 
 static guint bitmask[9] = { 0, 1, 3, 7, 15, 31, 63, 127, 255 };
@@ -767,15 +770,12 @@ _gdk_win32_colormap_color (GdkColormap *colormap,
     }
 }
 
-static void
+static COLORREF
 predraw_set_foreground (GdkGC       *gc,
 			GdkColormap *colormap,
 			gboolean    *ok)
 {
   COLORREF fg;
-  LOGBRUSH logbrush;
-  HPEN hpen;
-  HBRUSH hbr;
   GdkGCWin32 *win32_gc = (GdkGCWin32 *) gc;
   GdkColormapPrivateWin32 *colormap_private;
   gint k;
@@ -800,88 +800,59 @@ predraw_set_foreground (GdkGC       *gc,
   fg = _gdk_win32_colormap_color (colormap, win32_gc->foreground);
 
   GDK_NOTE (GC, g_print ("predraw_set_foreground: fg=%06lx\n", fg));
-
-  if (SetTextColor (win32_gc->hdc, fg) == CLR_INVALID)
-    WIN32_GDI_FAILED ("SetTextColor"), *ok = FALSE;
-
-  /* Create and select pen and brush. */
-
-  logbrush.lbStyle = BS_SOLID;
-  logbrush.lbColor = fg;
-  logbrush.lbHatch = 0;
-
-  if (win32_gc->pen_num_dashes > 0 && !IS_WIN_NT ())
-    {
-      /* The Win9x GDI is rather limited so we either draw dotted
-       * lines ourselve (only horizontal and vertical) or let them
-       * be drawn solid to avoid implementing a whole line renderer
-       */
-      if (*ok && (hpen = ExtCreatePen (
-                           (win32_gc->pen_style & ~(PS_STYLE_MASK)) | PS_SOLID,
-                           (win32_gc->pen_width > 0 ? win32_gc->pen_width : 1),
-                           &logbrush, 
-                           0, NULL)) == NULL)
-        WIN32_GDI_FAILED ("ExtCreatePen"), *ok = FALSE;
-    }
-  else
-    {
-      if (*ok && (hpen = ExtCreatePen (win32_gc->pen_style,
-                           (win32_gc->pen_width > 0 ? win32_gc->pen_width : 1),
-                           &logbrush, 
-                           win32_gc->pen_num_dashes,
-                           win32_gc->pen_dashes)) == NULL)
-        WIN32_GDI_FAILED ("ExtCreatePen"), *ok = FALSE;
-    }
-  
-  if (*ok && SelectObject (win32_gc->hdc, hpen) == NULL)
-    WIN32_GDI_FAILED ("SelectObject"), *ok = FALSE;
-
-  switch (win32_gc->fill_style)
-    {
-    case GDK_OPAQUE_STIPPLED:
-      if (*ok && (hbr = CreatePatternBrush (GDK_PIXMAP_HBITMAP (win32_gc->stipple))) == NULL)
-	WIN32_GDI_FAILED ("CreatePatternBrush"), *ok = FALSE;
-      if (*ok && !SetBrushOrgEx(win32_gc->hdc, gc->ts_x_origin,
-				gc->ts_y_origin, NULL))
-	WIN32_GDI_FAILED ("SetBrushOrgEx"), *ok = FALSE;
-	
-      break;
-
-    case GDK_SOLID:
-    default:
-      if (*ok && (hbr = CreateSolidBrush (fg)) == NULL)
-	WIN32_GDI_FAILED ("CreateSolidBrush"), *ok = FALSE;
-      break;
-  }
-  if (*ok)
-    {
-      HBRUSH old_hbr = SelectObject (win32_gc->hdc, hbr);
-      if (old_hbr == NULL)
-	WIN32_GDI_FAILED ("SelectObject"), *ok = FALSE;
-    }
-}  
-
-static void
-predraw_set_background (GdkGC       *gc,
-			GdkColormap *colormap,
-			gboolean    *ok)
-{
-  GdkGCWin32 *win32_gc = (GdkGCWin32 *) gc;
-
-  if (win32_gc->values_mask & GDK_GC_BACKGROUND)
-    {
-      COLORREF bg = _gdk_win32_colormap_color (colormap, win32_gc->background);
-
-      if (SetBkColor (win32_gc->hdc, bg) == CLR_INVALID)
-        WIN32_GDI_FAILED ("SetBkColor"), *ok = FALSE;
-    }
-  else
-    {
-      if (!SetBkMode (win32_gc->hdc, TRANSPARENT))
-        WIN32_GDI_FAILED ("SetBkMode"), *ok = FALSE;
-    }
+  return fg;
 }
 
+/**
+ * gdk_win32_hdc_get:
+ * @drawable: destination #GdkDrawable
+ * @gc: #GdkGC to use for drawing on @drawable
+ * @usage: mask indicating what properties needs to be set up
+ *
+ * Allocates a Windows device context handle (HDC) for drawing into
+ * @drawable, and sets it up appropriately according to @usage.
+ *
+ * Each #GdkGC can at one time have only one HDC associated with it.
+ *
+ * The following flags in @mask are handled:
+ *
+ * If %GDK_GC_FOREGROUND is set in @mask, a solid brush of the
+ * foreground color in @gc is selected into the HDC. The text color of
+ * the HDC is also set. If the @drawable has a palette (256-color
+ * mode), the palette is selected and realized.
+ *
+ * If any of the line attribute flags (%GDK_GC_LINE_WIDTH,
+ * %GDK_GC_LINE_STYLE, %GDK_GC_CAP_STYLE and %GDK_GC_JOIN_STYLE) is
+ * set in @mask, a solid pen of the foreground color and appropriate
+ * width and stule is created and selected into the HDC. Note that the
+ * dash properties are not completely implemented.
+ *
+ * If the %GDK_GC_FONT flag is set, the background mix mode is set to
+ * %TRANSPARENT. and the text alignment is set to
+ * %TA_BASELINE|%TA_LEFT. Note that no font gets selected into the HDC
+ * by this function.
+ *
+ * Some things are done regardless of @mask: If the function in @gc is
+ * any other than %GDK_COPY, the raster operation of the HDC is
+ * set. If @gc has a clip mask, the clip region of the HDC is set.
+ *
+ * Note that the fill style, tile, stipple, and tile and stipple
+ * origins in the @gc are ignored by this function. (In general, tiles
+ * and stipples can't be implemented directly on Win32; you need to do
+ * multiple pass drawing and blitting to implement tiles or
+ * stipples. GDK does just that when you call the GDK drawing
+ * functions with a GC that asks for tiles or stipples.)
+ *
+ * When the HDC is no longer used, it should be released by calling
+ * <function>gdk_win32_hdc_release()</function> with the same
+ * parameters.
+ *
+ * If you modify the HDC by calling <function>SelectObject</function>
+ * you should undo those modifications before calling
+ * <function>gdk_win32_hdc_release()</function>.
+ *
+ * Return value: The HDC.
+ **/
 HDC
 gdk_win32_hdc_get (GdkDrawable    *drawable,
 		   GdkGC          *gc,
@@ -890,6 +861,10 @@ gdk_win32_hdc_get (GdkDrawable    *drawable,
   GdkGCWin32 *win32_gc = (GdkGCWin32 *) gc;
   GdkDrawableImplWin32 *impl = NULL;
   gboolean ok = TRUE;
+  COLORREF fg = RGB (0, 0, 0);
+  LOGBRUSH logbrush;
+  HPEN hpen;
+  HBRUSH hbr;
 
   g_assert (win32_gc->hdc == NULL);
 
@@ -925,122 +900,96 @@ gdk_win32_hdc_get (GdkDrawable    *drawable,
     }
   
   if (ok && (usage & GDK_GC_FOREGROUND))
-    predraw_set_foreground (gc, impl->colormap, &ok);
+    {
+      fg = predraw_set_foreground (gc, impl->colormap, &ok);
+      if (ok && (hbr = CreateSolidBrush (fg)) == NULL)
+	WIN32_GDI_FAILED ("CreateSolidBrush"), ok = FALSE;
+
+      if (ok && SelectObject (win32_gc->hdc, hbr) == NULL)
+	WIN32_GDI_FAILED ("SelectObject"), ok = FALSE;
+
+      if (ok && SetTextColor (win32_gc->hdc, fg) == CLR_INVALID)
+	WIN32_GDI_FAILED ("SetTextColor"), ok = FALSE;
+    }
+
+  if (ok && (usage & LINE_ATTRIBUTES))
+    {
+      /* Create and select pen */
+      logbrush.lbStyle = BS_SOLID;
+      logbrush.lbColor = fg;
+      logbrush.lbHatch = 0;
+      
+      if (win32_gc->pen_num_dashes > 0 && !IS_WIN_NT ())
+	{
+	  /* The Win9x GDI is rather limited so we either draw dashed
+	   * lines ourselves (only horizontal and vertical) or let them be
+	   * drawn solid to avoid implementing a whole line renderer.
+	   */
+	  if ((hpen = ExtCreatePen (
+				    (win32_gc->pen_style & ~(PS_STYLE_MASK)) | PS_SOLID,
+				    MAX (win32_gc->pen_width, 1),
+				    &logbrush, 
+				    0, NULL)) == NULL)
+	    WIN32_GDI_FAILED ("ExtCreatePen"), ok = FALSE;
+	}
+      else
+	{
+	  if ((hpen = ExtCreatePen (win32_gc->pen_style,
+				    MAX (win32_gc->pen_width, 1),
+				    &logbrush, 
+				    win32_gc->pen_num_dashes,
+				    win32_gc->pen_dashes)) == NULL)
+	    WIN32_GDI_FAILED ("ExtCreatePen"), ok = FALSE;
+	}
+      
+      if (ok && SelectObject (win32_gc->hdc, hpen) == NULL)
+	WIN32_GDI_FAILED ("SelectObject"), ok = FALSE;
+    }
 
-  if (ok && (usage & GDK_GC_BACKGROUND))
-    predraw_set_background (gc, impl->colormap, &ok);
-  
   if (ok && (usage & GDK_GC_FONT))
     {
       if (SetBkMode (win32_gc->hdc, TRANSPARENT) == 0)
 	WIN32_GDI_FAILED ("SetBkMode"), ok = FALSE;
   
-      if (ok && SetTextAlign (win32_gc->hdc, TA_BASELINE) == GDI_ERROR)
+      if (ok && SetTextAlign (win32_gc->hdc, TA_BASELINE|TA_LEFT|TA_NOUPDATECP) == GDI_ERROR)
 	WIN32_GDI_FAILED ("SetTextAlign"), ok = FALSE;
     }
   
-  if (ok && (win32_gc->values_mask & GDK_GC_FUNCTION))
+  if (ok && win32_gc->rop2 != R2_COPYPEN)
     if (SetROP2 (win32_gc->hdc, win32_gc->rop2) == 0)
       WIN32_GDI_FAILED ("SetROP2"), ok = FALSE;
 
-  if (win32_gc->values_mask & GDK_GC_CLIP_MASK)
-    g_assert ((win32_gc->clip_region != NULL) != (win32_gc->hcliprgn != NULL));
-
-  if (ok
-      && (win32_gc->values_mask & GDK_GC_CLIP_MASK)
-      && win32_gc->clip_region != NULL)
-    {
-      HRGN hrgn;
-      RGNDATA *rgndata;
-      RECT *rect;
-      GdkRegionBox *boxes = win32_gc->clip_region->rects;
-      guint nbytes =
-	sizeof (RGNDATAHEADER) + (sizeof (RECT) * win32_gc->clip_region->numRects);
-      int i;
-
-      rgndata =	g_malloc (nbytes);
-      rgndata->rdh.dwSize = sizeof (RGNDATAHEADER);
-      rgndata->rdh.iType = RDH_RECTANGLES;
-      rgndata->rdh.nCount = rgndata->rdh.nRgnSize = 0;
-      SetRect (&rgndata->rdh.rcBound,
-	       G_MAXSHORT, G_MAXSHORT, G_MINSHORT, G_MINSHORT);
-
-      for (i = 0; i < win32_gc->clip_region->numRects; i++)
-	{
-	  rect = ((RECT *) rgndata->Buffer) + rgndata->rdh.nCount++;
-
-	  rect->left = CLAMP (boxes[i].x1 + gc->clip_x_origin,
-			      G_MINSHORT, G_MAXSHORT);
-	  rect->right = CLAMP (boxes[i].x2 + gc->clip_x_origin,
-			       G_MINSHORT, G_MAXSHORT);
-	  rect->top = CLAMP (boxes[i].y1 + gc->clip_y_origin,
-			     G_MINSHORT, G_MAXSHORT);
-	  rect->bottom = CLAMP (boxes[i].y2 + gc->clip_y_origin,
-				G_MINSHORT, G_MAXSHORT);
-
-	  GDK_NOTE (GC, g_print ("clip rgn box %d: %ldx%ld@+%ld+%ld\n",
-				 i,
-				 rect->right-rect->left,
-				 rect->bottom-rect->top,
-				 rect->left, rect->top));
-
-	  if (rect->left < rgndata->rdh.rcBound.left)
-	    rgndata->rdh.rcBound.left = rect->left;
-	  if (rect->right > rgndata->rdh.rcBound.right)
-	    rgndata->rdh.rcBound.right = rect->right;
-	  if (rect->top < rgndata->rdh.rcBound.top)
-	    rgndata->rdh.rcBound.top = rect->top;
-	  if (rect->bottom > rgndata->rdh.rcBound.bottom)
-	    rgndata->rdh.rcBound.bottom = rect->bottom;
-	}
-      if ((hrgn = ExtCreateRegion (NULL, nbytes, rgndata)) == NULL)
-	WIN32_API_FAILED ("ExtCreateRegion"), ok = FALSE;
-
-      g_free (rgndata);
-
-      /*
-       * XXX: combine the fillmode-stipple with the clip region possibly 
-       *  this needs to be done with the hcliprgn below as well, but 
-       *  the test case here were check boxes and radio buttons ...
-       */
-      if (   (win32_gc->values_mask & GDK_GC_STIPPLE)
-          && (win32_gc->values_mask & GDK_GC_FILL) && (win32_gc->fill_style == GDK_STIPPLED))
-        {
-          HRGN hstipplergn = _gdk_win32_bitmap_to_region (win32_gc->stipple);
-
-          if (OffsetRgn (hstipplergn, 
-             win32_gc->values_mask & GDK_GC_TS_X_ORIGIN ? gc->ts_x_origin : 0,
-             win32_gc->values_mask & GDK_GC_TS_Y_ORIGIN ? gc->ts_y_origin : 0) == ERROR)
-            WIN32_API_FAILED ("OffsetRgn"), ok = FALSE;
-          if (CombineRgn (hrgn, hrgn, hstipplergn, RGN_AND) == ERROR)
-            WIN32_API_FAILED ("CombineRgn"), ok = FALSE;
-          if (!DeleteObject (hstipplergn))
-            WIN32_API_FAILED ("DeleteObject");
-        }
-      if (ok && SelectClipRgn (win32_gc->hdc, hrgn) == ERROR)
-	WIN32_API_FAILED ("SelectClipRgn"), ok = FALSE;
-
-      if (hrgn != NULL)
-	DeleteObject (hrgn);
-    }
-  else if (ok
-	   && (win32_gc->values_mask & GDK_GC_CLIP_MASK)
-	   && win32_gc->hcliprgn != NULL)
+  if (ok &&
+      (win32_gc->values_mask & GDK_GC_CLIP_MASK) &&
+      win32_gc->hcliprgn != NULL)
     {
       if (SelectClipRgn (win32_gc->hdc, win32_gc->hcliprgn) == ERROR)
 	WIN32_API_FAILED ("SelectClipRgn"), ok = FALSE;
-      if (ok && !OffsetClipRgn (win32_gc->hdc,
-		 win32_gc->values_mask & GDK_GC_CLIP_X_ORIGIN ? gc->clip_x_origin : 0,
-		 win32_gc->values_mask & GDK_GC_CLIP_Y_ORIGIN ? gc->clip_y_origin : 0))
+
+      if (ok && win32_gc->values_mask & (GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN) &&
+	  OffsetClipRgn (win32_gc->hdc,
+	    win32_gc->values_mask & GDK_GC_CLIP_X_ORIGIN ? gc->clip_x_origin : 0,
+	    win32_gc->values_mask & GDK_GC_CLIP_Y_ORIGIN ? gc->clip_y_origin : 0) == ERROR)
 	WIN32_API_FAILED ("OffsetClipRgn"), ok = FALSE;
     }
 
-  GDK_NOTE (GC, (g_print ("gdk_win32_hdc_get: "),
-		 gdk_win32_print_dc (win32_gc->hdc)));
+  GDK_NOTE (GC, (g_print ("gdk_win32_hdc_get: %p (%s): ",
+			  win32_gc, _gdk_win32_gcvalues_mask_to_string (usage)),
+		 _gdk_win32_print_dc (win32_gc->hdc)));
 
   return win32_gc->hdc;
 }
 
+/**
+ * gdk_win32_hdc_release:
+ * @drawable: destination #GdkDrawable
+ * @gc: #GdkGC to use for drawing on @drawable
+ * @usage: mask indicating what properties were set up
+ *
+ * This function deallocates the Windows device context allocated by
+ * <funcion>gdk_win32_hdc_get()</function>. It should be called with
+ * the same parameters.
+ **/
 void
 gdk_win32_hdc_release (GdkDrawable    *drawable,
 		       GdkGC          *gc,
@@ -1051,7 +1000,9 @@ gdk_win32_hdc_release (GdkDrawable    *drawable,
   HGDIOBJ hpen = NULL;
   HGDIOBJ hbr = NULL;
 
-  GDK_NOTE (GC, g_print ("gdk_win32_hdc_release: %p\n", win32_gc->hdc));
+  GDK_NOTE (GC, g_print ("gdk_win32_hdc_release: %p: %p (%s)\n",
+			 win32_gc, win32_gc->hdc,
+			 _gdk_win32_gcvalues_mask_to_string (usage)));
 
   if (GDK_IS_DRAWABLE_IMPL_WIN32 (drawable))
     impl = GDK_DRAWABLE_IMPL_WIN32(drawable);
@@ -1076,36 +1027,26 @@ gdk_win32_hdc_release (GdkDrawable    *drawable,
       win32_gc->holdpal = NULL;
     }
 
+  if (usage & LINE_ATTRIBUTES)
+    if ((hpen = GetCurrentObject (win32_gc->hdc, OBJ_PEN)) == NULL)
+      WIN32_GDI_FAILED ("GetCurrentObject");
+  
   if (usage & GDK_GC_FOREGROUND)
-    {
-      if ((hpen = GetCurrentObject (win32_gc->hdc, OBJ_PEN)) == NULL)
-	WIN32_GDI_FAILED ("GetCurrentObject");
+    if ((hbr = GetCurrentObject (win32_gc->hdc, OBJ_BRUSH)) == NULL)
+      WIN32_GDI_FAILED ("GetCurrentObject");
 
-      if ((hbr = GetCurrentObject (win32_gc->hdc, OBJ_BRUSH)) == NULL)
-	WIN32_GDI_FAILED ("GetCurrentObject");
-    }
-
-  if (!RestoreDC (win32_gc->hdc, win32_gc->saved_dc))
-    WIN32_GDI_FAILED ("RestoreDC");
+  GDI_CALL (RestoreDC, (win32_gc->hdc, win32_gc->saved_dc));
 
   if (GDK_IS_PIXMAP_IMPL_WIN32 (impl))
-    {
-      if (!DeleteDC (win32_gc->hdc))
-	WIN32_GDI_FAILED ("DeleteDC");
-    }
+    GDI_CALL (DeleteDC, (win32_gc->hdc));
   else
-    {
-      if (!ReleaseDC (win32_gc->hwnd, win32_gc->hdc))
-	WIN32_GDI_FAILED ("ReleaseDC");
-    }
+    GDI_CALL (ReleaseDC, (win32_gc->hwnd, win32_gc->hdc));
 
   if (hpen != NULL)
-    if (!DeleteObject (hpen))
-      WIN32_GDI_FAILED ("DeleteObject");
+    GDI_CALL (DeleteObject, (hpen));
   
   if (hbr != NULL)
-    if (!DeleteObject (hbr))
-      WIN32_GDI_FAILED ("DeleteObject");
+    GDI_CALL (DeleteObject, (hbr));
 
   win32_gc->hdc = NULL;
 }
@@ -1117,12 +1058,12 @@ gdk_win32_hdc_release (GdkDrawable    *drawable,
  * bitmaps (and not pixmaps), with zero pixels being transparent.
  */
 
-/* _gdk_win32_bitmap_to_region : Create a region from the
+/* _gdk_win32_bitmap_to_hrgn : Create a region from the
  * "non-transparent" pixels of a bitmap.
  */
 
 HRGN
-_gdk_win32_bitmap_to_region (GdkPixmap *pixmap)
+_gdk_win32_bitmap_to_hrgn (GdkPixmap *pixmap)
 {
   HRGN hRgn = NULL;
   HRGN h;
@@ -1227,3 +1168,53 @@ _gdk_win32_bitmap_to_region (GdkPixmap *pixmap)
 
   return hRgn;
 }
+
+HRGN
+_gdk_win32_gdkregion_to_hrgn (GdkRegion *region,
+			      gint       x_origin,
+			      gint       y_origin)
+{
+  HRGN hrgn;
+  RGNDATA *rgndata;
+  RECT *rect;
+  GdkRegionBox *boxes = region->rects;
+  guint nbytes =
+    sizeof (RGNDATAHEADER) + (sizeof (RECT) * region->numRects);
+  int i;
+
+  rgndata = g_malloc (nbytes);
+  rgndata->rdh.dwSize = sizeof (RGNDATAHEADER);
+  rgndata->rdh.iType = RDH_RECTANGLES;
+  rgndata->rdh.nCount = rgndata->rdh.nRgnSize = 0;
+  SetRect (&rgndata->rdh.rcBound,
+	   G_MAXSHORT, G_MAXSHORT, G_MINSHORT, G_MINSHORT);
+
+  for (i = 0; i < region->numRects; i++)
+    {
+      rect = ((RECT *) rgndata->Buffer) + rgndata->rdh.nCount++;
+
+      rect->left = CLAMP (boxes[i].x1 + x_origin,
+			  G_MINSHORT, G_MAXSHORT);
+      rect->right = CLAMP (boxes[i].x2 + x_origin,
+			   G_MINSHORT, G_MAXSHORT);
+      rect->top = CLAMP (boxes[i].y1 + y_origin,
+			 G_MINSHORT, G_MAXSHORT);
+      rect->bottom = CLAMP (boxes[i].y2 + y_origin,
+			    G_MINSHORT, G_MAXSHORT);
+
+      if (rect->left < rgndata->rdh.rcBound.left)
+	rgndata->rdh.rcBound.left = rect->left;
+      if (rect->right > rgndata->rdh.rcBound.right)
+	rgndata->rdh.rcBound.right = rect->right;
+      if (rect->top < rgndata->rdh.rcBound.top)
+	rgndata->rdh.rcBound.top = rect->top;
+      if (rect->bottom > rgndata->rdh.rcBound.bottom)
+	rgndata->rdh.rcBound.bottom = rect->bottom;
+    }
+  if ((hrgn = ExtCreateRegion (NULL, nbytes, rgndata)) == NULL)
+    WIN32_API_FAILED ("ExtCreateRegion");
+
+  g_free (rgndata);
+
+  return (hrgn);
+}
