@@ -31,6 +31,7 @@
 #include "gdkevents.h"
 #include "gdkpixmap.h"
 #include "gdkwindow.h"
+#include "gdkdisplay.h"
 #include "gdkprivate-win32.h"
 #include "gdkinput-win32.h"
 
@@ -233,7 +234,7 @@ _gdk_windowing_window_init (void)
   impl = GDK_WINDOW_IMPL_WIN32 (private->impl);
   draw_impl = GDK_DRAWABLE_IMPL_WIN32 (private->impl);
   
-  draw_impl->handle = gdk_root_window;
+  draw_impl->handle = _gdk_root_window;
   draw_impl->wrapper = GDK_DRAWABLE (private);
   draw_impl->colormap = gdk_colormap_get_system ();
   gdk_colormap_ref (draw_impl->colormap);
@@ -245,7 +246,20 @@ _gdk_windowing_window_init (void)
 
   _gdk_window_init_position (GDK_WINDOW (private));
 
-  gdk_win32_handle_table_insert (&gdk_root_window, _gdk_parent_root);
+  gdk_win32_handle_table_insert (&_gdk_root_window, _gdk_parent_root);
+}
+
+static const gchar *
+get_default_title (void)
+{
+  const char *title;
+#if 0 /* So that we can build with GLib 2.0... */
+  title = g_get_application_name ();
+  if (!title)
+#endif
+    title = g_get_prgname ();
+
+  return title;
 }
 
 /* The Win API function AdjustWindowRect may return negative values
@@ -300,20 +314,20 @@ RegisterGdkClass (GdkWindowType wtype)
   wcl.lpfnWndProc = _gdk_win32_window_procedure;
   wcl.cbClsExtra = 0;
   wcl.cbWndExtra = 0;
-  wcl.hInstance = gdk_app_hmodule;
+  wcl.hInstance = _gdk_app_hmodule;
   wcl.hIcon = 0;
   /* initialize once! */
   if (0 == hAppIcon)
     {
       gchar sLoc [MAX_PATH+1];
 
-      if (0 != GetModuleFileName (gdk_app_hmodule, sLoc, MAX_PATH))
+      if (0 != GetModuleFileName (_gdk_app_hmodule, sLoc, MAX_PATH))
 	{
-	  hAppIcon = ExtractIcon (gdk_app_hmodule, sLoc, 0);
+	  hAppIcon = ExtractIcon (_gdk_app_hmodule, sLoc, 0);
 	  if (0 == hAppIcon)
 	    {
-	      if (0 != GetModuleFileName (gdk_dll_hinstance, sLoc, MAX_PATH))
-		hAppIcon = ExtractIcon (gdk_dll_hinstance, sLoc, 0);
+	      if (0 != GetModuleFileName (_gdk_dll_hinstance, sLoc, MAX_PATH))
+		hAppIcon = ExtractIcon (_gdk_dll_hinstance, sLoc, 0);
 	    }
 	}
       if (0 == hAppIcon) 
@@ -479,7 +493,7 @@ gdk_window_new (GdkWindow     *parent,
   if (attributes_mask & GDK_WA_TITLE)
     title = attributes->title;
   else
-    title = g_get_prgname ();
+    title = get_default_title ();
   if (!title || !*title)
     title = "GDK client window";
 
@@ -510,7 +524,7 @@ gdk_window_new (GdkWindow     *parent,
   else
     {
       dwExStyle = WS_EX_TRANSPARENT;
-      private->depth = visual->depth;
+      private->depth = 0;
       private->input_only = TRUE;
       draw_impl->colormap = gdk_colormap_get_system ();
       gdk_colormap_ref (draw_impl->colormap);
@@ -524,7 +538,7 @@ gdk_window_new (GdkWindow     *parent,
     {
     case GDK_WINDOW_TOPLEVEL:
       dwStyle = WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN;
-      hparent = gdk_root_window;
+      hparent = _gdk_root_window;
       break;
 
     case GDK_WINDOW_CHILD:
@@ -536,7 +550,7 @@ gdk_window_new (GdkWindow     *parent,
 #if 0
       dwExStyle |= WS_EX_TOPMOST; /* //HB: want this? */
 #endif
-      hparent = gdk_root_window;
+      hparent = _gdk_root_window;
       break;
 
     case GDK_WINDOW_TEMP:
@@ -596,7 +610,7 @@ gdk_window_new (GdkWindow     *parent,
 				      impl->position_info.width, impl->position_info.height,
 				      hparent,
 				      NULL,
-				      gdk_app_hmodule,
+				      _gdk_app_hmodule,
 				      NULL);
 #else
   {
@@ -609,7 +623,7 @@ gdk_window_new (GdkWindow     *parent,
 		    impl->position_info.width, impl->position_info.height,
 		    hparent,
 		    NULL,
-		    gdk_app_hmodule,
+		    _gdk_app_hmodule,
 		    window);
   if (GDK_WINDOW_HWND (window) != hwndNew)
     {
@@ -666,7 +680,8 @@ gdk_window_new (GdkWindow     *parent,
 }
 
 GdkWindow *
-gdk_window_foreign_new (GdkNativeWindow anid)
+gdk_window_foreign_new_for_display (GdkDisplay      *display,
+                                    GdkNativeWindow  anid)
 {
   GdkWindow *window;
   GdkWindowObject *private;
@@ -677,6 +692,8 @@ gdk_window_foreign_new (GdkNativeWindow anid)
   RECT rect;
   POINT point;
 
+  g_return_val_if_fail (display == gdk_display_get_default (), NULL);
+
   window = g_object_new (GDK_TYPE_WINDOW, NULL);
   private = (GdkWindowObject *)window;
   impl = GDK_WINDOW_IMPL_WIN32 (private->impl);
@@ -695,7 +712,7 @@ gdk_window_foreign_new (GdkNativeWindow anid)
   point.x = rect.left;
   point.y = rect.right;
   ClientToScreen ((HWND) anid, &point);
-  if (parent != gdk_root_window)
+  if (parent != _gdk_root_window)
     ScreenToClient (parent, &point);
   private->x = point.x;
   private->y = point.y;
@@ -737,7 +754,7 @@ _gdk_windowing_window_destroy (GdkWindow *window,
 			   GDK_WINDOW_HWND (window)));
 
   if (private->extension_events != 0)
-    gdk_input_window_destroy (window);
+    _gdk_input_window_destroy (window);
 
   if (private->window_type == GDK_WINDOW_FOREIGN)
     {
@@ -946,7 +963,9 @@ gdk_window_resize (GdkWindow *window,
 		   gint       width,
 		   gint       height)
 {
-  GdkWindowObject *private;
+  GdkWindowObject *private = (GdkWindowObject*) window;
+  GdkWindowImplWin32 *impl;
+  int x, y;
 
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
@@ -956,7 +975,7 @@ gdk_window_resize (GdkWindow *window,
   if (height < 1)
     height = 1;
 
-  private = (GdkWindowObject*) window;
+  impl = GDK_WINDOW_IMPL_WIN32 (private->impl);
   
   if (!GDK_WINDOW_DESTROYED (window))
     {
@@ -965,15 +984,10 @@ gdk_window_resize (GdkWindow *window,
 				       width, height);
       else
 	{
-	  GdkWindowImplWin32 *impl = GDK_WINDOW_IMPL_WIN32 (private->impl);
 	  POINT pt;
 	  RECT rect;
 	  DWORD dwStyle;
 	  DWORD dwExStyle;
-	  int x, y;
-
-	  if (width != impl->width || height != impl->height)
-	    private->resize_count += 1;
 
 	  pt.x = 0;
 	  pt.y = 0; 
@@ -997,6 +1011,7 @@ gdk_window_resize (GdkWindow *window,
 	                     SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOZORDER))
 	    WIN32_API_FAILED ("SetWindowPos");
 	}
+      private->resize_count += 1;
     }
 }
 
@@ -1007,7 +1022,8 @@ gdk_window_move_resize (GdkWindow *window,
 			gint       width,
 			gint       height)
 {
-  GdkWindowObject *private;
+  GdkWindowObject *private = (GdkWindowObject*) window;
+  GdkWindowImplWin32 *impl;
 
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
@@ -1017,10 +1033,14 @@ gdk_window_move_resize (GdkWindow *window,
   if (height < 1)
     height = 1;
   
-  private = (GdkWindowObject*) window;
+  impl = GDK_WINDOW_IMPL_WIN32 (private->impl);
 
-  if (!GDK_WINDOW_DESTROYED (window))
+  if (!private->destroyed)
     {
+      RECT rect;
+      DWORD dwStyle;
+      DWORD dwExStyle;
+
       GDK_NOTE (MISC, g_print ("gdk_window_move_resize: %p %dx%d@+%d+%d\n",
 			       GDK_WINDOW_HWND (window),
 			       width, height, x, y));
@@ -1029,14 +1049,6 @@ gdk_window_move_resize (GdkWindow *window,
 	_gdk_window_move_resize_child (window, x, y, width, height);
       else
 	{
-	  GdkWindowImplWin32 *impl = GDK_WINDOW_IMPL_WIN32 (private->impl);
-	  RECT rect;
-	  DWORD dwStyle;
-	  DWORD dwExStyle;
-
-	  if (width != impl->width || height != impl->height)
-	    private->resize_count += 1;
-	  
 	  rect.left = x;
 	  rect.top = y;
 	  rect.right = x + width;
@@ -1547,7 +1559,7 @@ gdk_window_set_background (GdkWindow *window,
   
   GDK_NOTE (MISC, g_print ("gdk_window_set_background: %p %s\n",
 			   GDK_WINDOW_HWND (window), 
-			   gdk_win32_color_to_string (color)));
+			   _gdk_win32_color_to_string (color)));
 
   private->bg_color = *color;
 
@@ -1570,6 +1582,7 @@ gdk_window_set_back_pixmap (GdkWindow *window,
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   g_return_if_fail (pixmap == NULL || !parent_relative);
+  g_return_if_fail (pixmap == NULL || gdk_drawable_get_depth (window) == gdk_drawable_get_depth (pixmap));
   
   if (private->bg_pixmap &&
       private->bg_pixmap != GDK_PARENT_RELATIVE_BG &&
@@ -1843,7 +1856,8 @@ gdk_window_get_frame_extents (GdkWindow    *window,
 }
 
 GdkWindow*
-_gdk_windowing_window_get_pointer (GdkWindow       *window,
+_gdk_windowing_window_get_pointer (GdkDisplay      *display,
+				   GdkWindow       *window,
 				   gint            *x,
 				   gint            *y,
 				   GdkModifierType *mask)
@@ -1851,21 +1865,17 @@ _gdk_windowing_window_get_pointer (GdkWindow       *window,
   GdkWindow *return_val;
   POINT screen_point, point;
   HWND hwnd, hwndc;
+  BYTE kbd[256];
 
   g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
   
-  if (!window)
-    window = _gdk_parent_root;
-
   return_val = NULL;
   GetCursorPos (&screen_point);
   point = screen_point;
   ScreenToClient (GDK_WINDOW_HWND (window), &point);
 
-  if (x)
-    *x = point.x;
-  if (y)
-    *y = point.y;
+  *x = point.x;
+  *y = point.y;
 
   hwnd = WindowFromPoint (point);
   if (hwnd != NULL)
@@ -1890,35 +1900,44 @@ _gdk_windowing_window_get_pointer (GdkWindow       *window,
   else
     return_val = NULL;
       
-  if (mask)
-    {
-      BYTE kbd[256];
-
-      GetKeyboardState (kbd);
-      *mask = 0;
-      if (kbd[VK_SHIFT] & 0x80)
-	*mask |= GDK_SHIFT_MASK;
-      if (kbd[VK_CAPITAL] & 0x80)
-	*mask |= GDK_LOCK_MASK;
-      if (kbd[VK_CONTROL] & 0x80)
-	*mask |= GDK_CONTROL_MASK;
-      if (kbd[VK_MENU] & 0x80)
-	*mask |= GDK_MOD1_MASK;
-      if (kbd[VK_LBUTTON] & 0x80)
-	*mask |= GDK_BUTTON1_MASK;
-      if (kbd[VK_MBUTTON] & 0x80)
-	*mask |= GDK_BUTTON2_MASK;
-      if (kbd[VK_RBUTTON] & 0x80)
-	*mask |= GDK_BUTTON3_MASK;
-    }
+  GetKeyboardState (kbd);
+  *mask = 0;
+  if (kbd[VK_SHIFT] & 0x80)
+    *mask |= GDK_SHIFT_MASK;
+  if (kbd[VK_CAPITAL] & 0x80)
+    *mask |= GDK_LOCK_MASK;
+  if (kbd[VK_CONTROL] & 0x80)
+    *mask |= GDK_CONTROL_MASK;
+  if (kbd[VK_MENU] & 0x80)
+    *mask |= GDK_MOD1_MASK;
+  if (kbd[VK_LBUTTON] & 0x80)
+    *mask |= GDK_BUTTON1_MASK;
+  if (kbd[VK_MBUTTON] & 0x80)
+    *mask |= GDK_BUTTON2_MASK;
+  if (kbd[VK_RBUTTON] & 0x80)
+    *mask |= GDK_BUTTON3_MASK;
   
   return return_val;
 }
 
+void
+_gdk_windowing_get_pointer (GdkDisplay       *display,
+			    GdkScreen       **screen,
+			    gint             *x,
+			    gint             *y,
+			    GdkModifierType  *mask)
+{
+  GdkScreen *default_screen = gdk_display_get_default_screen (display);
+  GdkWindow *root_window = gdk_screen_get_root_window (default_screen);
+  
+  *screen = default_screen;
+  _gdk_windowing_window_get_pointer (display, root_window, x, y, mask);
+}
+
 GdkWindow*
-_gdk_windowing_window_at_pointer (GdkScreen *screen,
-				  gint      *win_x,
-				  gint      *win_y)
+_gdk_windowing_window_at_pointer (GdkDisplay *display,
+				  gint       *win_x,
+				  gint       *win_y)
 {
   GdkWindow *window;
   POINT point, pointc;
@@ -1932,10 +1951,8 @@ _gdk_windowing_window_at_pointer (GdkScreen *screen,
   if (hwnd == NULL)
     {
       window = _gdk_parent_root;
-      if (win_x)
-	*win_x = pointc.x;
-      if (win_y)
-	*win_y = pointc.y;
+      *win_x = pointc.x;
+      *win_y = pointc.y;
       return window;
     }
       
@@ -1952,10 +1969,8 @@ _gdk_windowing_window_at_pointer (GdkScreen *screen,
   if (window && (win_x || win_y))
     {
       GetClientRect (hwnd, &rect);
-      if (win_x)
-	*win_x = point.x - rect.left;
-      if (win_y)
-	*win_y = point.y - rect.top;
+      *win_x = point.x - rect.left;
+      *win_y = point.y - rect.top;
     }
 
   GDK_NOTE (MISC, g_print ("gdk_window_at_pointer: +%ld+%ld %p%s\n",
@@ -2442,6 +2457,20 @@ gdk_window_unmaximize (GdkWindow *window)
 				 0);
 }
 
+void
+gdk_window_fullscreen (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  g_warning ("gdk_window_fullscreen() not implemented.\n");
+}
+
+void
+gdk_window_unfullscreen (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+}
+
 void
 gdk_window_focus (GdkWindow *window,
                   guint32    timestamp)
@@ -2479,6 +2508,20 @@ gdk_window_set_modal_hint (GdkWindow *window,
       WIN32_API_FAILED ("SetWindowPos");
 }
 
+void
+gdk_window_set_skip_taskbar_hint (GdkWindow *window,
+				  gboolean   skips_taskbar)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+}
+
+void
+gdk_window_set_skip_pager_hint (GdkWindow *window,
+				gboolean   skips_pager)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+}
+
 void
 gdk_window_set_type_hint (GdkWindow        *window,
 			  GdkWindowTypeHint hint)
@@ -2554,3 +2597,11 @@ gdk_window_begin_move_drag (GdkWindow *window,
 
   /* XXX: isn't all this default on win32 ... */  
 }
+
+GdkWindow *
+gdk_window_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
+{
+  g_return_val_if_fail (display == gdk_display_get_default(), NULL);
+
+  return gdk_window_lookup (anid);
+}
