@@ -44,13 +44,18 @@
 #define MENU_ITEM_CLASS(w)   GTK_MENU_ITEM_GET_CLASS (w)
 #define	MENU_NEEDS_RESIZE(m) GTK_MENU_SHELL (m)->menu_flag
 
-#define SUBMENU_NAV_REGION_PADDING 2
-#define SUBMENU_NAV_HYSTERESIS_TIMEOUT 333
+#define DEFAULT_POPUP_DELAY    225
+#define DEFAULT_POPDOWN_DELAY  1000
 
-#define MENU_SCROLL_STEP 10
+#define NAVIGATION_REGION_OVERSHOOT 50  /* How much the navigation region
+					 * extends below the submenu
+					 */
+
+#define MENU_SCROLL_STEP1 8
+#define MENU_SCROLL_STEP2 15
 #define MENU_SCROLL_ARROW_HEIGHT 16
-#define MENU_SCROLL_FAST_ZONE 4
-#define MENU_SCROLL_TIMEOUT1 150
+#define MENU_SCROLL_FAST_ZONE 8
+#define MENU_SCROLL_TIMEOUT1 50
 #define MENU_SCROLL_TIMEOUT2 50
 
 typedef struct _GtkMenuAttachData	GtkMenuAttachData;
@@ -99,6 +104,10 @@ static gboolean gtk_menu_expose            (GtkWidget        *widget,
 					    GdkEventExpose   *event);
 static gboolean gtk_menu_key_press         (GtkWidget        *widget,
 					    GdkEventKey      *event);
+static gboolean gtk_menu_button_press      (GtkWidget        *widget,
+					    GdkEventButton   *event);
+static gboolean gtk_menu_button_release    (GtkWidget        *widget,
+					    GdkEventButton   *event);
 static gboolean gtk_menu_motion_notify     (GtkWidget        *widget,
 					    GdkEventMotion   *event);
 static gboolean gtk_menu_enter_notify      (GtkWidget        *widget,
@@ -107,8 +116,11 @@ static gboolean gtk_menu_leave_notify      (GtkWidget        *widget,
 					    GdkEventCrossing *event);
 static void     gtk_menu_scroll_to         (GtkMenu          *menu,
 					    gint              offset);
-static void     gtk_menu_stop_scrolling    (GtkMenu          *menu);
-static gboolean gtk_menu_scroll_timeout    (gpointer          data);
+
+static void     gtk_menu_stop_scrolling        (GtkMenu  *menu);
+static void     gtk_menu_remove_scroll_timeout (GtkMenu  *menu);
+static gboolean gtk_menu_scroll_timeout        (gpointer  data);
+
 static void     gtk_menu_scroll_item_visible (GtkMenuShell    *menu_shell,
 					      GtkWidget       *menu_item);
 static void     gtk_menu_select_item       (GtkMenuShell     *menu_shell,
@@ -126,6 +138,7 @@ static void     gtk_menu_style_set         (GtkWidget        *widget,
 					    GtkStyle         *previous_style);
 static gboolean gtk_menu_focus             (GtkWidget        *widget,
 					    GtkDirectionType direction);
+static gint     gtk_menu_get_popup_delay   (GtkMenuShell    *menu_shell);
 
 
 static void     gtk_menu_stop_navigating_submenu       (GtkMenu          *menu);
@@ -241,11 +254,14 @@ gtk_menu_class_init (GtkMenuClass *class)
   widget_class->show = gtk_menu_show;
   widget_class->expose_event = gtk_menu_expose;
   widget_class->key_press_event = gtk_menu_key_press;
+  widget_class->button_press_event = gtk_menu_button_press;
+  widget_class->button_release_event = gtk_menu_button_release;
   widget_class->motion_notify_event = gtk_menu_motion_notify;
   widget_class->show_all = gtk_menu_show_all;
   widget_class->hide_all = gtk_menu_hide_all;
   widget_class->enter_notify_event = gtk_menu_enter_notify;
   widget_class->leave_notify_event = gtk_menu_leave_notify;
+  widget_class->motion_notify_event = gtk_menu_motion_notify;
   widget_class->style_set = gtk_menu_style_set;
   widget_class->focus = gtk_menu_focus;
 
@@ -255,6 +271,7 @@ gtk_menu_class_init (GtkMenuClass *class)
   menu_shell_class->deactivate = gtk_menu_deactivate;
   menu_shell_class->select_item = gtk_menu_select_item;
   menu_shell_class->insert = gtk_menu_real_insert;
+  menu_shell_class->get_popup_delay = gtk_menu_get_popup_delay;
 
   binding_set = gtk_binding_set_by_class (class);
   gtk_binding_entry_add_signal (binding_set,
@@ -303,6 +320,23 @@ gtk_menu_class_init (GtkMenuClass *class)
 						       _("Whether menu accelerators can be changed by pressing a key over the menu item"),
 						       FALSE,
 						       G_PARAM_READWRITE));
+
+  gtk_settings_install_property (g_param_spec_int ("gtk-menu-popup-delay",
+						   _("Delay before submenus appear"),
+						   _("Minimum time the pointer must stay over a menu item before the submenu appear"),
+						   0,
+						   G_MAXINT,
+						   DEFAULT_POPUP_DELAY,
+						   G_PARAM_READWRITE));
+
+  gtk_settings_install_property (g_param_spec_int ("gtk-menu-popdown-delay",
+						   _("Delay before hiding a submenu"),
+						   _("The time before hiding a submenu when the pointer is moving towards the submenu"),
+						   0,
+						   G_MAXINT,
+						   DEFAULT_POPDOWN_DELAY,
+						   G_PARAM_READWRITE));
+						   
 }
 
 
@@ -495,6 +529,19 @@ gtk_menu_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+attach_widget_screen_changed (GtkWidget *attach_widget,
+			      GdkScreen *previous_screen,
+			      GtkMenu   *menu)
+{
+  if (gtk_widget_has_screen (attach_widget) &&
+      !g_object_get_data (G_OBJECT (menu), "gtk-menu-explicit-screen"))
+    {
+      gtk_window_set_screen (GTK_WINDOW (menu->toplevel),
+			     gtk_widget_get_screen (attach_widget));
+    }
+}
+
 void
 gtk_menu_attach_to_widget (GtkMenu	       *menu,
 			   GtkWidget	       *attach_widget,
@@ -514,7 +561,7 @@ gtk_menu_attach_to_widget (GtkMenu	       *menu,
     {
       g_warning ("gtk_menu_attach_to_widget(): menu already attached to %s",
 		 g_type_name (G_TYPE_FROM_INSTANCE (data->attach_widget)));
-      return;
+     return;
     }
   
   g_object_ref (menu);
@@ -522,6 +569,11 @@ gtk_menu_attach_to_widget (GtkMenu	       *menu,
   
   data = g_new (GtkMenuAttachData, 1);
   data->attach_widget = attach_widget;
+  
+  g_signal_connect (attach_widget, "screen_changed",
+		    G_CALLBACK (attach_widget_screen_changed), menu);
+  attach_widget_screen_changed (attach_widget, NULL, menu);
+  
   data->detacher = detacher;
   g_object_set_data (G_OBJECT (menu), attach_data_key, data);
   
@@ -566,6 +618,10 @@ gtk_menu_detach (GtkMenu *menu)
     }
   g_object_set_data (G_OBJECT (menu), attach_data_key, NULL);
   
+  g_signal_handlers_disconnect_by_func (data->attach_widget,
+					(gpointer) attach_widget_screen_changed,
+					menu);
+
   data->detacher (data->attach_widget, menu);
   
   if (GTK_WIDGET_REALIZED (menu))
@@ -614,7 +670,7 @@ gtk_menu_real_insert (GtkMenuShell     *menu_shell,
 {
   if (GTK_WIDGET_REALIZED (menu_shell))
     gtk_widget_set_parent_window (child, GTK_MENU (menu_shell)->bin_window);
-  
+
   GTK_MENU_SHELL_CLASS (parent_class)->insert (menu_shell, child, position);
 }
 
@@ -724,7 +780,6 @@ gtk_menu_popup (GtkMenu		    *menu,
   GtkWidget *parent;
   GdkEvent *current_event;
   GtkMenuShell *menu_shell;
-  GtkMenuAttachData *attach_data;
 
   g_return_if_fail (GTK_IS_MENU (menu));
   
@@ -733,21 +788,6 @@ gtk_menu_popup (GtkMenu		    *menu,
   
   menu_shell->parent_menu_shell = parent_menu_shell;
   
-  if (!g_object_get_data (G_OBJECT (menu), "gtk-menu-explicit-screen"))
-    {
-      /* The screen was not set explicitly, if the menu is
-       * attached to a widget, try to get screen from its 
-       * toplevel window else go with the default
-       */
-      attach_data = g_object_get_data (G_OBJECT (menu), attach_data_key);
-      if (attach_data)
-	{
-	  if (!GTK_WIDGET_REALIZED (menu))
-	  gtk_window_set_screen (GTK_WINDOW (menu->toplevel),
-				 gtk_widget_get_screen (attach_data->attach_widget));
-	}
-    }
-
   /* Find the last viewable ancestor, and make an X grab on it
    */
   parent = GTK_WIDGET (menu);
@@ -1225,7 +1265,7 @@ gtk_menu_set_tearoff_state (GtkMenu  *menu,
 		GTK_ADJUSTMENT (gtk_adjustment_new (0,
 						    0,
 						    GTK_WIDGET (menu)->requisition.height,
-						    MENU_SCROLL_STEP,
+						    MENU_SCROLL_STEP2,
 						    height/2,
 						    height));
 	      g_object_connect (menu->tearoff_adjustment,
@@ -1819,6 +1859,40 @@ gtk_menu_show (GtkWidget *widget)
   GTK_WIDGET_CLASS (parent_class)->show (widget);
 }
 
+static gboolean
+gtk_menu_button_press (GtkWidget      *widget,
+			 GdkEventButton *event)
+{
+  /* Don't pop down the menu for releases over scroll arrows
+   */
+  if (GTK_IS_MENU (widget))
+    {
+      GtkMenu *menu = GTK_MENU (widget);
+
+      if (menu->upper_arrow_prelight ||  menu->lower_arrow_prelight)
+	return TRUE;
+    }
+
+  return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
+}
+
+static gboolean
+gtk_menu_button_release (GtkWidget      *widget,
+			 GdkEventButton *event)
+{
+  /* Don't pop down the menu for releases over scroll arrows
+   */
+  if (GTK_IS_MENU (widget))
+    {
+      GtkMenu *menu = GTK_MENU (widget);
+
+      if (menu->upper_arrow_prelight ||  menu->lower_arrow_prelight)
+	return TRUE;
+    }
+
+  return GTK_WIDGET_CLASS (parent_class)->button_release_event (widget, event);
+}
+
 static gboolean
 gtk_menu_key_press (GtkWidget	*widget,
 		    GdkEventKey *event)
@@ -1969,7 +2043,7 @@ gtk_menu_motion_notify  (GtkWidget	   *widget,
 
   if (GTK_IS_MENU (widget))
     gtk_menu_handle_scrolling (GTK_MENU (widget), TRUE);
-  
+
   /* We received the event for one of two reasons:
    *
    * a) We are the active menu, and did gtk_grab_add()
@@ -2122,15 +2196,14 @@ gtk_menu_handle_scrolling (GtkMenu *menu, gboolean enter)
 	  /* Deselect the active item so that any submenus are poped down */
 	  gtk_menu_shell_deselect (menu_shell);
 
-	  gtk_menu_stop_scrolling (menu);
-	  menu->scroll_step = -MENU_SCROLL_STEP;
+	  gtk_menu_remove_scroll_timeout (menu);
+	  menu->scroll_step = (scroll_fast) ? -MENU_SCROLL_STEP2 : -MENU_SCROLL_STEP1;
 	  menu->timeout_id = g_timeout_add ((scroll_fast) ? MENU_SCROLL_TIMEOUT2 : MENU_SCROLL_TIMEOUT1,
 					    gtk_menu_scroll_timeout,
 					    menu);
 	}
       else if (!enter && !in_arrow && menu->upper_arrow_prelight)
 	{
-	  menu->upper_arrow_prelight = FALSE;
 	  gdk_window_invalidate_rect (GTK_WIDGET (menu)->window, &rect, FALSE);
 	  
 	  gtk_menu_stop_scrolling (menu);
@@ -2162,15 +2235,14 @@ gtk_menu_handle_scrolling (GtkMenu *menu, gboolean enter)
 	  /* Deselect the active item so that any submenus are poped down */
 	  gtk_menu_shell_deselect (menu_shell);
 
-	  gtk_menu_stop_scrolling (menu);
-	  menu->scroll_step = MENU_SCROLL_STEP;
+	  gtk_menu_remove_scroll_timeout (menu);
+	  menu->scroll_step = (scroll_fast) ? MENU_SCROLL_STEP2 : MENU_SCROLL_STEP1;
 	  menu->timeout_id = g_timeout_add ((scroll_fast) ? MENU_SCROLL_TIMEOUT2 : MENU_SCROLL_TIMEOUT1,
 					    gtk_menu_scroll_timeout,
 					    menu);
 	}
       else if (!enter && !in_arrow && menu->lower_arrow_prelight)
 	{
-	  menu->lower_arrow_prelight = FALSE;
 	  gdk_window_invalidate_rect (GTK_WIDGET (menu)->window, &rect, FALSE);
 	  
 	  gtk_menu_stop_scrolling (menu);
@@ -2185,8 +2257,13 @@ gtk_menu_enter_notify (GtkWidget        *widget,
   GtkWidget *menu_item;
 
   if (widget && GTK_IS_MENU (widget))
-    gtk_menu_handle_scrolling (GTK_MENU (widget), TRUE);
-      
+    {
+      GtkMenuShell *menu_shell = GTK_MENU_SHELL (widget);
+
+      if (!menu_shell->ignore_enter)
+	gtk_menu_handle_scrolling (GTK_MENU (widget), TRUE);
+    }
+  
   /* If this is a faked enter (see gtk_menu_motion_notify), 'widget'
    * will not correspond to the event widget's parent.  Check to see
    * if we are in the parent's navigation region.
@@ -2310,6 +2387,71 @@ gtk_menu_navigating_submenu (GtkMenu *menu,
   return FALSE;
 }
 
+#undef DRAW_STAY_UP_TRIANGLE
+
+#ifdef DRAW_STAY_UP_TRIANGLE
+
+static void
+draw_stay_up_triangle (GdkWindow *window,
+		       GdkRegion *region)
+{
+  /* Draw ugly color all over the stay-up triangle */
+  GdkColor ugly_color = { 0, 50000, 10000, 10000 };
+  GdkGCValues gc_values;
+  GdkGC *ugly_gc;
+  GdkRectangle clipbox;
+
+  gc_values.subwindow_mode = GDK_INCLUDE_INFERIORS;
+  ugly_gc = gdk_gc_new_with_values (window, &gc_values, 0 | GDK_GC_SUBWINDOW);
+  gdk_gc_set_rgb_fg_color (ugly_gc, &ugly_color);
+  gdk_gc_set_clip_region (ugly_gc, region);
+
+  gdk_region_get_clipbox (region, &clipbox);
+  
+  gdk_draw_rectangle (window,
+                     ugly_gc,
+                     TRUE,
+                     clipbox.x, clipbox.y,
+                     clipbox.width, clipbox.height);
+  
+  g_object_unref (G_OBJECT (ugly_gc));
+}
+#endif
+
+static GdkRegion *
+flip_region (GdkRegion *region,
+	     gboolean   flip_x,
+	     gboolean   flip_y)
+{
+  gint n_rectangles;
+  GdkRectangle *rectangles;
+  GdkRectangle clipbox;
+  GdkRegion *new_region;
+  gint i;
+
+  new_region = gdk_region_new ();
+  
+  gdk_region_get_rectangles (region, &rectangles, &n_rectangles);
+  gdk_region_get_clipbox (region, &clipbox);
+
+  for (i = 0; i < n_rectangles; ++i)
+    {
+      GdkRectangle rect = rectangles[i];
+
+      if (flip_y)
+	rect.y -= 2 * (rect.y - clipbox.y) + rect.height;
+
+      if (flip_x)
+	rect.x -= 2 * (rect.x - clipbox.x) + rect.width;
+
+      gdk_region_union_with_rect (new_region, &rect);
+    }
+
+  g_free (rectangles);
+
+  return new_region;
+}
+
 static void
 gtk_menu_set_submenu_navigation_region (GtkMenu          *menu,
 					GtkMenuItem      *menu_item,
@@ -2331,6 +2473,7 @@ gtk_menu_set_submenu_navigation_region (GtkMenu          *menu,
   
   gdk_window_get_origin (menu_item->submenu->window, &submenu_left, &submenu_top);
   gdk_drawable_get_size (menu_item->submenu->window, &width, &height);
+  
   submenu_right = submenu_left + width;
   submenu_bottom = submenu_top + height;
   
@@ -2338,48 +2481,71 @@ gtk_menu_set_submenu_navigation_region (GtkMenu          *menu,
   
   if (event->x >= 0 && event->x < width)
     {
-      /* Set navigation region */
-      /* We fudge/give a little padding in case the user
-       * ``misses the vertex'' of the triangle/is off by a pixel or two.
-       */ 
-      if (menu_item->submenu_direction == GTK_DIRECTION_RIGHT)
-	point[0].x = event->x_root - SUBMENU_NAV_REGION_PADDING; 
-      else                             
-	point[0].x = event->x_root + SUBMENU_NAV_REGION_PADDING;  
+      gint popdown_delay;
+      gboolean flip_y = FALSE;
+      gboolean flip_x = FALSE;
       
-      /* Exiting the top or bottom? */ 
+      gtk_menu_stop_navigating_submenu (menu);
+
+      if (menu_item->submenu_direction == GTK_DIRECTION_RIGHT)
+	{
+	  /* right */
+	  point[0].x = event->x_root;
+	  point[1].x = submenu_left;
+	}
+      else
+	{
+	  /* left */
+	  point[0].x = event->x_root + 1;
+	  point[1].x = 2 * (event->x_root + 1) - submenu_right;
+
+	  flip_x = TRUE;
+	}
+
       if (event->y < 0)
-        { /* top */
+	{
+	  /* top */
 	  point[0].y = event->y_root + 1;
-	  point[1].y = submenu_top;
+	  point[1].y = 2 * (event->y_root + 1) - submenu_top + NAVIGATION_REGION_OVERSHOOT;
 
-	  if (point[0].y <= point[1].y)
+	  if (point[0].y >= point[1].y - NAVIGATION_REGION_OVERSHOOT)
 	    return;
+
+	  flip_y = TRUE;
 	}
       else
-        { /* bottom */
+	{
+	  /* bottom */
 	  point[0].y = event->y_root;
-	  point[1].y = submenu_bottom;
+	  point[1].y = submenu_bottom + NAVIGATION_REGION_OVERSHOOT;
 
-	  if (point[0].y >= point[1].y)
+	  if (point[0].y >= submenu_bottom)
 	    return;
 	}
-      
-      /* Submenu is to the left or right? */ 
-      if (menu_item->submenu_direction == GTK_DIRECTION_RIGHT)
-	point[1].x = submenu_left;  /* right */
-      else
-	point[1].x = submenu_right; /* left */
-      
+
       point[2].x = point[1].x;
       point[2].y = point[0].y;
 
-      gtk_menu_stop_navigating_submenu (menu);
-      
       menu->navigation_region = gdk_region_polygon (point, 3, GDK_WINDING_RULE);
 
-      menu->navigation_timeout = gtk_timeout_add (SUBMENU_NAV_HYSTERESIS_TIMEOUT, 
+      if (flip_x || flip_y)
+	{
+	  GdkRegion *new_region = flip_region (menu->navigation_region, flip_x, flip_y);
+	  gdk_region_destroy (menu->navigation_region);
+	  menu->navigation_region = new_region;
+	}
+
+      g_object_get (G_OBJECT (gtk_widget_get_settings (GTK_WIDGET (menu))),
+		    "gtk-menu-popdown-delay", &popdown_delay,
+		    NULL);
+
+      menu->navigation_timeout = gtk_timeout_add (popdown_delay,
 						  gtk_menu_stop_navigating_submenu_cb, menu);
+
+#ifdef DRAW_STAY_UP_TRIANGLE
+      draw_stay_up_triangle (gdk_get_default_root_window(),
+			     menu->navigation_region);
+#endif
     }
 }
 
@@ -2422,6 +2588,8 @@ gtk_menu_position (GtkMenu *menu)
 
   screen = gtk_widget_get_screen (widget);
   monitor_num = gdk_screen_get_monitor_at_point (screen, x, y);
+  if (monitor_num < 0)
+    monitor_num = 0;
   gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
   /* We need the requisition to figure out the right place to
@@ -2498,7 +2666,7 @@ gtk_menu_position (GtkMenu *menu)
 }
 
 static void
-gtk_menu_stop_scrolling (GtkMenu *menu)
+gtk_menu_remove_scroll_timeout (GtkMenu *menu)
 {
   if (menu->timeout_id)
     {
@@ -2507,6 +2675,14 @@ gtk_menu_stop_scrolling (GtkMenu *menu)
     }
 }
 
+static void
+gtk_menu_stop_scrolling (GtkMenu *menu)
+{
+  gtk_menu_remove_scroll_timeout (menu);
+
+  menu->upper_arrow_prelight = FALSE;
+  menu->lower_arrow_prelight = FALSE;
+}
 
 static void
 gtk_menu_scroll_to (GtkMenu *menu,
@@ -2538,12 +2714,6 @@ gtk_menu_scroll_to (GtkMenu *menu,
   view_height -= (border_width + widget->style->ythickness) * 2;
   menu_height = widget->requisition.height - (border_width + widget->style->ythickness) * 2;
 
-  offset = CLAMP (offset, 0, menu_height - view_height);
-
-  /* Scroll the menu: */
-  if (GTK_WIDGET_REALIZED (menu))
-    gdk_window_move (menu->bin_window, 0, -offset);
-
   x = border_width + widget->style->xthickness;
   y = border_width + widget->style->ythickness;
   
@@ -2585,6 +2755,12 @@ gtk_menu_scroll_to (GtkMenu *menu,
 	y += MENU_SCROLL_ARROW_HEIGHT;
     }
 
+  offset = CLAMP (offset, 0, menu_height - view_height);
+
+  /* Scroll the menu: */
+  if (GTK_WIDGET_REALIZED (menu))
+    gdk_window_move (menu->bin_window, 0, -offset);
+
   if (GTK_WIDGET_REALIZED (menu))
     gdk_window_move_resize (menu->view_window,
 			    x,
@@ -2777,18 +2953,43 @@ gtk_menu_hide_all (GtkWidget *widget)
 /**
  * gtk_menu_set_screen:
  * @menu: a #GtkMenu.
- * @screen: a #GtkScreen.
+ * @screen: a #GdkScreen, or %NULL if the screen should be
+ *          determined by the widget the menu is attached to.
  *
- * Sets the #GtkScreen on which the GtkMenu will be displayed.
- * This function can only be called before @menu is realized.
+ * Sets the #GdkScreen on which the menu will be displayed.
+ * 
+ * Since: 2.2
  **/
 void
-gtk_menu_set_screen (GtkMenu *menu, 
+gtk_menu_set_screen (GtkMenu   *menu, 
 		     GdkScreen *screen)
 {
   g_return_if_fail (GTK_IS_MENU (menu));
-  gtk_window_set_screen (GTK_WINDOW (menu->toplevel), 
-			 screen);
+  g_return_if_fail (!screen || GDK_IS_SCREEN (screen));
+
   g_object_set_data (G_OBJECT (menu), "gtk-menu-explicit-screen", screen);
+
+  if (screen)
+    {
+      gtk_window_set_screen (GTK_WINDOW (menu->toplevel), screen);
+    }
+  else
+    {
+      GtkWidget *attach_widget = gtk_menu_get_attach_widget (menu);
+      if (attach_widget)
+	attach_widget_screen_changed (attach_widget, NULL, menu);
+    }
 }
 
+
+static gint
+gtk_menu_get_popup_delay (GtkMenuShell *menu_shell)
+{
+  gint popup_delay;
+
+  g_object_get (G_OBJECT (gtk_widget_get_settings (GTK_WIDGET (menu_shell))),
+		"gtk-menu-popup-delay", &popup_delay,
+		NULL);
+
+  return popup_delay;
+}
