@@ -57,7 +57,6 @@
 
 #include "gtkmain.h"
 #include "gtkselection.h"
-#include "gtksignal.h"
 
 /* #define DEBUG_SELECTION */
 
@@ -301,12 +300,14 @@ gtk_target_list_find (GtkTargetList *list,
  * @display: the #Gdkdisplay where the selection is set 
  * @widget: new selection owner (a #GdkWidget), or %NULL.
  * @selection: an interned atom representing the selection to claim.
- * @time: timestamp with which to claim the selection
+ * @time_: timestamp with which to claim the selection
  *
  * Claim ownership of a given selection for a particular widget, or,
  * if @widget is %NULL, release ownership of the selection.
  *
  * Return value: TRUE if the operation succeeded 
+ * 
+ * Since: 2.2
  */
 gboolean
 gtk_selection_owner_set_for_display (GdkDisplay   *display,
@@ -380,14 +381,15 @@ gtk_selection_owner_set_for_display (GdkDisplay   *display,
        */
       if (old_owner && old_owner != widget)
 	{
-	  GdkEventSelection event;
+	  GdkEvent *event = gdk_event_new (GDK_SELECTION_CLEAR);
 	  
-	  event.type = GDK_SELECTION_CLEAR;
-	  event.window = old_owner->window;
-	  event.selection = selection;
-	  event.time = time;
+	  event->selection.window = g_object_ref (old_owner->window);
+	  event->selection.selection = selection;
+	  event->selection.time = time;
 	  
-	  gtk_widget_event (old_owner, (GdkEvent *) &event);
+	  gtk_widget_event (old_owner, event);
+
+	  gdk_event_free (event);
 	}
       return TRUE;
     }
@@ -399,7 +401,7 @@ gtk_selection_owner_set_for_display (GdkDisplay   *display,
  * gtk_selection_owner_set:
  * @widget:  a #GtkWidget, or %NULL.
  * @selection:  an interned atom representing the selection to claim
- * @time: timestamp with which to claim the selection
+ * @time_: timestamp with which to claim the selection
  * 
  * Claims ownership of a given selection for a particular widget,
  * or, if @widget is %NULL, release ownership of the selection.
@@ -457,7 +459,7 @@ gtk_selection_target_list_get (GtkWidget    *widget,
   GList *tmp_list;
   GList *lists;
 
-  lists = gtk_object_get_data (GTK_OBJECT (widget), gtk_selection_handler_key);
+  lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
   tmp_list = lists;
   while (tmp_list)
@@ -473,7 +475,7 @@ gtk_selection_target_list_get (GtkWidget    *widget,
   sellist->list = gtk_target_list_new (NULL, 0);
 
   lists = g_list_prepend (lists, sellist);
-  gtk_object_set_data (GTK_OBJECT (widget), gtk_selection_handler_key, lists);
+  g_object_set_data (G_OBJECT (widget), gtk_selection_handler_key, lists);
 
   return sellist->list;
 }
@@ -485,7 +487,7 @@ gtk_selection_target_list_remove (GtkWidget    *widget)
   GList *tmp_list;
   GList *lists;
 
-  lists = gtk_object_get_data (GTK_OBJECT (widget), gtk_selection_handler_key);
+  lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
   tmp_list = lists;
   while (tmp_list)
@@ -499,7 +501,7 @@ gtk_selection_target_list_remove (GtkWidget    *widget)
     }
 
   g_list_free (lists);
-  gtk_object_set_data (GTK_OBJECT (widget), gtk_selection_handler_key, NULL);
+  g_object_set_data (G_OBJECT (widget), gtk_selection_handler_key, NULL);
 }
 
 /**
@@ -518,7 +520,7 @@ gtk_selection_clear_targets (GtkWidget *widget,
   GList *tmp_list;
   GList *lists;
 
-  lists = gtk_object_get_data (GTK_OBJECT (widget), gtk_selection_handler_key);
+  lists = g_object_get_data (G_OBJECT (widget), gtk_selection_handler_key);
   
   tmp_list = lists;
   while (tmp_list)
@@ -536,7 +538,7 @@ gtk_selection_clear_targets (GtkWidget *widget,
       tmp_list = tmp_list->next;
     }
   
-  gtk_object_set_data (GTK_OBJECT (widget), gtk_selection_handler_key, lists);
+  g_object_set_data (G_OBJECT (widget), gtk_selection_handler_key, lists);
 }
 
 void 
@@ -1024,6 +1026,8 @@ gtk_selection_init (void)
  * be deprecated in future versions of GTK+.
  * 
  * Return value: %TRUE if the event was handled, otherwise false
+ * 
+ * Since: 2.2
  **/
 gboolean
 gtk_selection_clear (GtkWidget         *widget,
@@ -1733,9 +1737,9 @@ gtk_selection_retrieval_report (GtkRetrievalInfo *info,
   data.data = buffer;
   data.display = gtk_widget_get_display (info->widget);
   
-  gtk_signal_emit_by_name (GTK_OBJECT(info->widget),
-			   "selection_received", 
-			   &data, time);
+  g_signal_emit_by_name (info->widget,
+			 "selection_received", 
+			 &data, time);
 }
 
 /*************************************************************
@@ -1768,10 +1772,10 @@ gtk_selection_invoke_handler (GtkWidget	       *widget,
   if (target_list && 
       gtk_target_list_find (target_list, data->target, &info))
     {
-      gtk_signal_emit_by_name (GTK_OBJECT (widget), 
-			       "selection_get",
-			       data,
-			       info, time);
+      g_signal_emit_by_name (widget,
+			     "selection_get",
+			     data,
+			     info, time);
     }
   else
     gtk_selection_default_handler (widget, data);
